(************************************************************************
 *                                                                      *
 *   Ant Movie Catalog 3.x                                              *
 *   (C) 2000-2011 Antoine Potten, Mickal Vanneufville                 *
 *   http://www.antp.be/software                                        *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 *   This file can be used freely in any program, even if it is not     *
 *   opensource or if it is commercial. It can be used only to provide  *
 *   compatibility with Ant Movie Catalog files, for importation for    *
 *   example. A mention to the origin of this code and eventually       *
 *   a link to Ant Movie Catalog website somewhere in the about box,    *
 *   help file or documentation would be appreciated.                   *
 *   Like for the GPL-licensed files, this file is distributed WITHOUT  *
 *   ANY WARRANTY.                                                      *
 *                                                                      *
 *   To compile fields.pas and movieclass.pas in any project,           *
 *   you have to define "DLLMode" either in project options or          *
 *   directly in these two files.                                       *
 *                                                                      *
 *   Alternatively, this can be used under GPL license:                 *
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 2     *
 *   of the License, or (at your option) any later version.             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 ************************************************************************)

unit movieclass;

interface

uses
  Classes, Contnrs, SysUtils {$IFNDEF DLLMode},

  JvSimpleXml, Dialogs,

  movieclass_old{$ENDIF}, Fields, Interfaces;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

const
  strClipboardHeader = 'AMC35';
  intFileVersion     = 35;

type
  TMovieIncludeOption = (mioAll, mioSelected, mioChecked, mioVisible);
  TMovieDuplicateNumbers = (mdnIgnore, mdnSwap, mdnShift);

  TMovie = class(TInterfacedObject, IMovie)
  private
{$IFNDEF DLLMode}
    mutex: Cardinal;
{$ENDIF}
  public
    iNumber: Integer;
    strMedia: string;
    strMediaType: string;
    strSource: string;
    iDate: integer;
    strBorrower: string;
    iRating: Integer;
    strOriginalTitle: string;
    strTranslatedTitle: string;
    strDirector: string;
    strProducer: string;
    strCountry: string;
    strCategory: string;
    iYear: Integer;
    iLength: Integer;
    strActors: string;
    strURL: string;
    strDescription: string;
    strComments: string;
    strVideoFormat: string;
    iVideoBitrate: Integer;
    strAudioFormat: string;
    iAudioBitrate: Integer;
    strResolution: string;
    strFramerate: string;
    strLanguages: string;
    strSubtitles: string;
    strSize: string;
    iDisks: Integer;
    bChecked: Boolean;
    strPicture: string;
    Picture: TMemoryStream;
    _listItems: TObjectList; // List of TElTreeItem associated to the movie (Updated in RefreshMovieList)
    _bSelected: Boolean; // Movie selected or not (Updated in StoreStates/LoadStates)
    _bVisible: Boolean; // Movie visible or not(Updated in StoreStates/LoadStates)
    _selectedGroup: string; // Group where movie is selected (Updated in StoreStates/LoadStates)
    _selectedItem: TObject; // TElTreeItem selected (Updated in StoreStates/LoadStates)
    _thumb: TMemoryStream;
    _thumbError: Word;
    _thumbWidth: Word;
    _thumbHeight: Word;

    constructor Create;
    destructor  Destroy;override;
{$IFNDEF DLLMode}
    procedure Lock();
    function LockWait(msTime: Integer): boolean;
    procedure Unlock();
    procedure GetFields(MovieFields: PMovie30);
    procedure SetFields(MovieFields: PMovie30);
{$ENDIF}
    procedure WriteData(const OutputFile: TStream);
    procedure ReadData(const InputFile: TStream; const Version: Integer = 99);
{$IFDEF MSWINDOWS}
    function  SaveToMemory: THandle;
    class procedure FreeMemory(DataHandle: THandle; OnlyUnlock: Boolean = False);
    procedure LoadFromMemory(DataHandle: THandle);
{$ENDIF}
{$IFNDEF DLLMode}
    procedure SaveToXML(Root: TJvSimpleXmlElem; const strFileName: string = '');
    procedure LoadFromXML(Root: TJvSimpleXmlElem; const Version: Integer = 99);
    function  GetFieldValue(const FieldName: string): string; overload;
    procedure SetFieldValue(const FieldName: string; const Value: string); overload;
{$ENDIF}
    function  GetFieldValue(const FieldID: TMovieField; const LocalFormatSettings: Boolean = False): string; overload;
    function  GetIntFieldValue(const FieldID: TMovieField): Integer;
    procedure SetFieldValue(const FieldID: TMovieField; const Value: string); overload;
    procedure InitFields;
{$IFNDEF DLLMode}
    function  GetFormattedTitle: string; overload;
    function  GetFormattedTitle(DisplayType: Integer; UsePrefixes: Boolean): string; overload;
{$ENDIF}
    procedure Assign(AMovie: TMovie; IncludeNumber: Boolean = True);
    function CanInclude(const IncOpt: TMovieIncludeOption): Boolean;
    function CalcTotalSize: Int64;
    function ContainsText(const Value: string; const Field: Integer; const WholeField: Boolean): Boolean;
{$IFNDEF DLLMode}
    function GetPictureSize(const CatalogDir: TFileName = ''): Int64;
{$ENDIF}
  end;

  TMovieProperties = class(TObject)
  public
    strEncoding: string;
    strName: string;
    strMail: string;
    strSite: string;
    strDescription: string;
    procedure InitFields;
  end;

  TMovieFilter = class(TObject)
  public
    field: Integer; // -1 = AllFields
    value: string;
    wholeField: Boolean;
  end;

  TMovieList = class(TInterfacedObjectList, IMovieList)
  private
    procedure ReadDatas(InputFile: TFileStream; const version: integer);
{$IFNDEF DLLMode}
    procedure ReadRecords(InputFile: TFileStream; const RecordSize: longint; const version: integer);
    procedure ReadPictures(const strFilename: string);
    procedure ReadBorrowers(const strFilename: string);
{$ENDIF}
    function GetItem(const idx: Integer): TMovie;
    function GetItem2(const idx: Integer): IMovie;
    procedure SetItem(const idx: Integer; const Value: TMovie);
  public
    MovieProperties: TMovieProperties;
    constructor Create(AOwnsObjects: Boolean = True); reintroduce; overload;
    destructor Destroy; override;
    function Find(const MovieNumber: Integer): TMovie;
    procedure SaveToFile(const strFilename: string);
    procedure LoadFromFile(const strFileName: string);
    class function ReadHeader(const AFileName: TFileName; out TheHeader: string): Int64;
{$IFNDEF DLLMode}
    procedure SaveToXML(const strFilename: string; ConvertPictures: Boolean{; const Header: string});
    procedure LoadFromXML(const strFilename: string);
    procedure Sort(const FieldsList: TStrings); overload;
    procedure SortReverse(const FieldsList: TStrings); overload;
    function GetMoviesByGroups(const Field: Integer; const IncOpt: TMovieIncludeOption = mioALL;
      MovieFilter : TMovieFilter = nil): TStringList;
{$ENDIF}
    procedure Sort(const Field: Integer); overload;
    procedure SortReverse(const Field: Integer); overload;
    procedure Add(const AMovieList: TMovieList; const AllowDuplicateNumbers: Boolean); overload;
    function Add: TMovie; overload;
    function Add2: IMovie; overload;
    function MaxNumber(ListIsSorted: Boolean = False): Integer;
    function FirstFreeNumber(ListIsSorted: Boolean = False): Integer;
    function HasImages(IncludeStored, IncludeLinksRel, IncludeLinksAbs: Boolean): Boolean;
    procedure GetValues(AList: TStrings; Field: Integer);
    function Count: Integer; overload;
    function Count(const IncOpt: TMovieIncludeOption): Integer; overload;
    procedure Count(out cAll, cSelected, cChecked, cVisible: Integer); overload;
    function Count(const MovieNumber: Integer): Integer; overload;
    procedure MakeFilteredList(AList: TList; const IncOpt: TMovieIncludeOption);
    procedure ShiftNumbers(const StartAt: Integer; const Exclude: TMovie);
    property Items[const idx: Integer]: TMovie read GetItem write SetItem; default;
end;

TFieldType = (ftInteger, ftReal, ftDate, ftString, ftList, ftOther);

function GetFieldType(field: Integer): TFieldType;
function CompareStd(Item1, Item2: Pointer): Integer;
function CompareStdReverse(Item1, Item2: Pointer): Integer;
{$IFNDEF DLLMode}
function CompareAdv(Item1, Item2: Pointer): Integer;
function CompareAdvReverse(Item1, Item2: Pointer): Integer;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

var
  ClipboardFormat: WORD;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

implementation

uses
  IniFiles{$IFDEF MSWINDOWS}, Windows{$ENDIF}, DateUtils, Math, StrUtils

  {$IFNDEF DLLMode}, functions_str, Global, functions_files,
  functions_xml, ConstValues, ProgramSettings{$ENDIF};

const
  IntSize = SizeOf(Integer);
  BoolSize = SizeOf(Boolean);

const
  strErrorSave = 'Unable to save file "%s": ';
  strErrorLoad = 'Unable to load file "%s": ';
  strErrorHeader = 'Bad header';
  strUnableToIdentify = 'Cannot identify file type from header';
  strNoMovieFound = 'No catalog found in this file';

  strFileHeader   = ' AMC_?.? Ant Movie Catalog         www.buypin.com  www.ant.be.tf ';
  strFileHeader31 = ' AMC_3.1 Ant Movie Catalog 3.1.x   www.buypin.com  www.ant.be.tf ';
  strFileHeader33 = ' AMC_3.3 Ant Movie Catalog 3.3.x   www.buypin.com  www.ant.be.tf ';
  strFileHeader35 = ' AMC_3.5 Ant Movie Catalog 3.5.x   www.buypin.com    www.antp.be ';
  strFileHeader36 = ' AMC_3.6 Ant Movie Catalog 3.6.x   www.buypin.com    www.antp.be ';

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

var
{$IFNDEF DLLMode}
  SortFieldsList: TStringList;
  SortFieldsIndex: Integer;
{$ENDIF}
  SortField: Integer;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

constructor TMovie.Create;
begin
  inherited create;
{$IFNDEF DLLMode}
  mutex := CreateMutex(nil, False, Pchar('TMovieMutex'+IntToStr(Integer(Self))));
{$ENDIF}
  _listItems := TObjectList.Create(False);
  Picture := nil;
  _thumb := nil;
  _thumbError := 0;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

destructor TMovie.Destroy;
begin
  if (Picture <> nil) then
    Picture.Free;
  if (_thumb <> nil) then
    _thumb.Free;
  _listItems.Free;
{$IFNDEF DLLMode}
  CloseHandle(mutex);
{$ENDIF}
  inherited destroy;
end;

{$IFNDEF DLLMode}
procedure TMovie.Lock();
begin
  if WaitForSingleObject(mutex, INFINITE) <> WAIT_OBJECT_0 then
    RaiseLastOSError;
end;

function TMovie.LockWait(msTime: Integer): boolean;
begin
  if WaitForSingleObject(mutex, msTime) <> WAIT_OBJECT_0 then
    Result := false
  else
    Result := true;
end;

procedure TMovie.Unlock();
begin
  ReleaseMutex(Mutex);
end;

procedure TMovie.GetFields(MovieFields: PMovie30);
begin
  if MovieFields <> nil then
  begin
    FillChar(MovieFields^,sizeof(MovieFields^),' ');
    with MovieFields^ do
    begin
      iNumber := self.iNumber;
      strOriginalTitle := self.strOriginalTitle;
      strTranslatedTitle := self.strTranslatedTitle;
      strDirector := self.strDirector;
      strProducer := self.strProducer;
      strCountry := self.strCountry;
      iYear := self.iYear;
      strCategory := self.strCategory;
      iLength := self.iLength;
      strActors := self.strActors;
      strURL := self.strURL;
      StrLCopy(@(strDescription)[1],PChar(self.strDescription),sizeof(strDescription));
      strComments := self.strComments;
      strVideoFormat := self.strVideoFormat;
      strSize := self.strSize;
      strResolution := self.strResolution;
      strLanguages := self.strLanguages;
      strSubtitles := self.strSubtitles;
      iRating := self.iRating;
      bChecked := self.bChecked;
      iDate := self.iDate;
      strPicture := self.strPicture;
      if Picture = nil then
        iPictureSize := 0
      else
        iPictureSize := Picture.Size;
      strBorrower := self.strBorrower;
    end;
  end;
end;

procedure TMovie.SetFields(MovieFields: PMovie30);
begin
  self.InitFields;
  if MovieFields <> nil then
  begin
    with MovieFields^ do
    begin
      self.iNumber := iNumber;
      self.strOriginalTitle := strOriginalTitle;
      self.strTranslatedTitle := strTranslatedTitle;
      self.strDirector := strDirector;
      self.strProducer := strProducer;
      self.strCountry := strCountry;
      if iYear = 1 then
        self.iYear := -1
      else
        self.iYear := iYear;
      self.strCategory := strCategory;
      if iLength = 1 then
        self.iLength := -1
      else
        self.iLength := iLength;
      self.strActors := strActors;
      if strURL = 'http://' then
        self.strURL := ''
      else
        self.strURL := strURL;
      self.strDescription := strDescription;
      self.strComments := strComments;
      self.strVideoFormat := strVideoFormat;
      self.strSize := strSize;
      self.strResolution := strResolution;
      self.strLanguages := strLanguages;
      self.strSubtitles := strSubtitles;
      self.iRating := iRating;
      self.bChecked := bChecked;
      self.iDate := iDate;
      self.strPicture := strPicture;
      self.strBorrower := strBorrower;
    end;
  end;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovie.WriteData(const OutputFile: TStream);
var
  recsize: integer;

  procedure WriteField(const field: string);
  begin
    with OutputFile do
    begin
      recsize := Length(field);
      WriteBuffer(recsize, intsize);
      if recsize > 0 then
        WriteBuffer(field[1], recsize);
    end;
  end;

begin
  with OutputFile do
  begin
    WriteBuffer(iNumber, intsize);
    WriteBuffer(iDate, intsize);
    WriteBuffer(iRating, intsize);
    WriteBuffer(iYear, intsize);
    WriteBuffer(iLength, intsize);
    WriteBuffer(iVideoBitrate, intsize);
    WriteBuffer(iAudioBitrate, intsize);
    WriteBuffer(iDisks, intsize);
    WriteBuffer(bChecked, boolsize);
    WriteField(strMedia);
    WriteField(strMediaType);
    WriteField(strSource);
    WriteField(strBorrower);
    WriteField(strOriginalTitle);
    WriteField(strTranslatedTitle);
    WriteField(strDirector);
    WriteField(strProducer);
    WriteField(strCountry);
    WriteField(strCategory);
    WriteField(strActors);
    WriteField(strURL);
    WriteField(strDescription);
    WriteField(strComments);
    WriteField(strVideoFormat);
    WriteField(strAudioFormat);
    WriteField(strResolution);
    WriteField(strFramerate);
    WriteField(strLanguages);
    WriteField(strSubtitles);
    WriteField(strSize);
    WriteField(strPicture);
    if (Picture = nil) then
      recsize := 0
    else
      recsize := Picture.Size;
    WriteBuffer(recsize, intsize);
    if recsize > 0 then
    begin
      Picture.Seek(0, soFromBeginning);
      CopyFrom(Picture, recsize)
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovie.ReadData(const InputFile: TStream; const Version: Integer = 99);
var
  recsize: integer;

  function ReadField: string;
  begin
    with InputFile do
    begin
      ReadBuffer(recsize, intsize);
      if recsize > 0 then
      begin
        SetLength(Result, recsize);
        ReadBuffer(Result[1], recsize);
      end else
        Result := '';
    end;
  end;

begin
  with InputFile do
  begin
    ReadBuffer(iNumber, intsize);
    ReadBuffer(iDate, intsize);
    ReadBuffer(iRating, intsize);
    if (Version < 35) and (iRating <> -1) then
      iRating := iRating * 10;
    ReadBuffer(iYear, intsize);
    ReadBuffer(iLength, intsize);
    ReadBuffer(iVideoBitrate, intsize);
    ReadBuffer(iAudioBitrate, intsize);
    ReadBuffer(iDisks, intsize);
    ReadBuffer(bChecked, boolsize);
    strMedia := ReadField;
    if Version >= 33 then
    begin
      strMediaType := ReadField;
      strSource := ReadField;
    end else
    begin
      strMediaType := '';
      strSource := '';
    end;
    strBorrower := ReadField;
    strOriginalTitle := ReadField;
    strTranslatedTitle := ReadField;
    strDirector := ReadField;
    strProducer := ReadField;
    strCountry := ReadField;
    strCategory := ReadField;
    strActors := ReadField;
    strURL := ReadField;
    strDescription := ReadField;
    strComments := ReadField;
    strVideoFormat := ReadField;
    strAudioFormat := ReadField;
    strResolution := ReadField;
    strFramerate := ReadField;
    strLanguages := ReadField;
    strSubtitles := ReadField;
    strSize := ReadField;
    strPicture := ReadField;
    ReadBuffer(recsize, intsize);
    if recsize > 0 then
    begin
      Picture := TMemoryStream.Create;
      Picture.CopyFrom(InputFile, recsize)
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFDEF MSWINDOWS}
function TMovie.SaveToMemory: THandle;
var
  Stream: TMemoryStream;
  DataPtr: Pointer;
begin
  Stream := TMemoryStream.Create;
  try
    WriteData(Stream);
    Stream.Seek(0, soFromBeginning);
    Result := GlobalAlloc(GMEM_MOVEABLE, Stream.Size);
    if Result <> 0 then
    begin
      DataPtr := GlobalLock(Result);
      if DataPtr <> nil then
      begin
        Move(Stream.Memory^, DataPtr^, Stream.Size);
      end;
    end;
  finally
    Stream.Free;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

class procedure TMovie.FreeMemory(DataHandle: THandle; OnlyUnlock: Boolean = False);
begin
  if DataHandle <> 0 then
  begin
    GlobalUnlock(DataHandle);
    if not OnlyUnlock then
      GlobalFree(DataHandle);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovie.LoadFromMemory(DataHandle: THandle);
var
  DataPtr: Pointer;
  DataSize: Integer;
  Stream: TMemoryStream;
begin
  if DataHandle <> 0 then
  begin
    DataPtr := GlobalLock(DataHandle);
    if DataPtr <> nil then
    begin
      Stream := TMemoryStream.Create;
      try
        DataSize := GlobalSize(DataHandle);
        Stream.SetSize(DataSize);
        Move(DataPtr^, stream.Memory^, DataSize);
        Stream.Seek(0, soFromBeginning);
        ReadData(Stream);
      finally
        Stream.Free;
        GlobalUnlock(DataHandle);
      end;
    end;
  end;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
function TMovie.GetFieldValue(const FieldName: string):string;
begin
  Result := GetFieldValue(strFields.IndexOf(FieldName));
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovie.GetFieldValue(const FieldID: TMovieField; const LocalFormatSettings: Boolean = False): string;
begin
  case FieldID of
    fieldNumber:          Result := IntToStr(iNumber);
    fieldChecked:         Result := BoolToStr(bChecked, True);
    fieldMedia:           Result := strMedia;
    fieldMediaType:       Result := strMediaType;
    fieldSource:          Result := strSource;
    fieldDate:
        if iDate > 0 then
        begin
          Result := DateToStr(iDate);
        end else
          Result := '';
    fieldBorrower:        Result := strBorrower;
    fieldRating:
      if iRating = -1 then
        Result := ''
      else
      begin
        {$IFNDEF DLLMode}
        if Settings.rOptions.rMovieInformation.RatingTrunc then
          Result := IntToStr(iRating div 10)
        else
          if not LocalFormatSettings then
            Result := FormatFloat('#0.0', iRating / 10, FormatSettings)
          else
        {$ENDIF}
            Result := FormatFloat('#0.0', iRating / 10);
      end;
    fieldOriginalTitle:   Result := strOriginalTitle;
    fieldTranslatedTitle: Result := strTranslatedTitle;
    {$IFNDEF DLLMode}
    fieldFormattedTitle:  Result := GetFormattedTitle(Settings.rOptions.rMovieList.TitleColumn, Settings.rOptions.rMovieList.UsePrefixes);
    {$ENDIF}
    fieldDirector:        Result := strDirector;
    fieldProducer:        Result := strProducer;
    fieldCountry:         Result := strCountry;
    fieldCategory:        Result := strCategory;
    fieldYear:
        if iYear = -1 then
          Result := ''
        else
          Result := IntToStr(iYear);
    fieldLength:
        if iLength = -1 then
          Result := ''
        else
          Result := IntToStr(iLength);
    fieldActors:          Result := strActors;
    fieldURL:             Result := strURL;
    fieldDescription:     Result := strDescription;
    fieldComments:        Result := strComments;
    fieldVideoFormat:     Result := strVideoFormat;
    fieldAudioFormat:     Result := strAudioFormat;
    fieldVideoBitrate:
        if iVideoBitrate = -1 then
          Result := ''
        else
          Result := IntToStr(iVideoBitrate);
    fieldAudioBitrate:
        if iAudioBitrate = -1 then
          Result := ''
        else
          Result := IntToStr(iAudioBitrate);
    fieldResolution:      Result := strResolution;
    fieldFramerate:       Result := strFramerate;
    fieldLanguages:       Result := strLanguages;
    fieldSubtitles:       Result := strSubtitles;
    fieldSize:            Result := strSize;
    fieldDisks:
        if iDisks = -1 then
          Result := ''
        else
          Result := IntToStr(iDisks);
  else
    Result := '';
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovie.GetIntFieldValue(const FieldID: TMovieField): Integer;
begin
  case FieldID of
    fieldNumber:          Result := iNumber;
    fieldDate:            Result := iDate;
    fieldRating:          Result := iRating;
    fieldYear:            Result := iYear;
    fieldLength:          Result := iLength;
    fieldVideoBitrate:    Result := iVideoBitrate;
    fieldAudioBitrate:    Result := iAudioBitrate;
    fieldDisks:           Result := iDisks;
  else
    Result := -1;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
procedure TMovie.SetFieldValue(const FieldName: string; const Value: string);
var
  i: Integer;
begin
  if FieldName = strFieldPicture then
  begin
    if strPicture = '' then
      strPicture := Value;
  end
  else
  begin
    i := strFields.IndexOf(FieldName);
    if i in AllFields then
      SetFieldValue(i, Value);
  end;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovie.SetFieldValue(const FieldID: TMovieField; const Value: string);
begin
  case FieldID of
    fieldNumber:          iNumber := StrToIntDef(Value, 0);
    fieldChecked:         bChecked := not ((Value = '') or (Value = '0') or SameText(Value, 'False'));
    fieldMedia:           strMedia := Value;
    fieldMediaType:       strMediaType := Value;
    fieldSource:          strSource := Value;
    fieldBorrower:        strBorrower := value;
    fieldDate:            iDate := trunc(StrToDateDef(Value, 0));
    {$IFNDEF DLLMode}
    fieldRating:          iRating := Trunc(StrToFloatDef(CharReplace(Value, DecimalSeparator, '.'), - 0.1, FormatSettings) * 10);
    {$ELSE}
    fieldRating:          iRating := StrToIntDef(Value, -1);
    {$ENDIF}
    fieldOriginalTitle:   strOriginalTitle := Value;
    fieldTranslatedTitle: strTranslatedTitle := Value;
    fieldDirector:        strDirector := Value;
    fieldProducer:        strProducer := Value;
    fieldCountry:         strCountry := Value;
    fieldCategory:        strCategory := Value;
    fieldYear:            iYear := StrToIntDef(Value, -1);
    fieldLength:          iLength := StrToIntDef(Value, -1);
    fieldActors:          strActors := Value;
    fieldURL:             strURL := Value;
    fieldDescription:     strDescription := Value;
    fieldComments:        strComments := Value;
    fieldVideoFormat:     strVideoFormat := Value;
    fieldAudioFormat:     strAudioFormat := Value;
    fieldVideoBitrate:    iVideoBitrate := StrToIntDef(Value, -1);
    fieldAudioBitrate:    iAudioBitrate := StrToIntDef(Value, -1);
    fieldResolution:      strResolution := Value;
    fieldFrameRate:       strFramerate := Value;
    fieldLanguages:       strLanguages := Value;
    fieldSubtitles:       strSubtitles := Value;
    fieldSize:            strSize := Value;
    fieldDisks:           iDisks := StrToIntDef(Value, -1);
  end; // case
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovie.InitFields;
begin
  iNumber := 0;
  bChecked := True;
  strMedia := '';
  strMediaType := '';
  strSource := '';
  iDate := 0;
  strBorrower := '';
  iRating := -1;
  strOriginalTitle := '';
  strTranslatedTitle := '';
  strDirector := '';
  strProducer := '';
  strCountry := '';
  strCategory := '';
  iYear := -1;
  iLength := -1;
  strActors := '';
  strURL := '';
  strDescription := '';
  strComments := '';
  strVideoFormat := '';
  iVideoBitrate := -1;
  strAudioFormat := '';
  iAudioBitrate := -1;
  strResolution := '';
  strFramerate := '';
  strLanguages := '';
  strSubtitles := '';
  strSize := '';
  iDisks := -1;
  strPicture := '';
  Picture := nil;
  _bSelected := True;
  _bVisible := True;
  _thumb := nil;
  _thumbError := 0;
  _thumbWidth := 0;
  _thumbHeight := 0;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}
{$IFNDEF DLLMode}

function TMovie.GetFormattedTitle: string;
begin
  with Settings.rOptions.rMovieList do
    Result := Global.GetFormattedTitle(strOriginalTitle, strTranslatedTitle, strMedia, TitleColumn, UsePrefixes);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovie.GetFormattedTitle(DisplayType: Integer; UsePrefixes: Boolean): string;
begin
  Result := Global.GetFormattedTitle(strOriginalTitle, strTranslatedTitle, strMedia, DisplayType, UsePrefixes);
end;

{$ENDIF}
{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovie.Assign(AMovie: TMovie; IncludeNumber: Boolean = True);
begin
  if IncludeNumber then
    iNumber := AMovie.iNumber;
  strMedia := AMovie.strMedia;
  strMediaType := AMovie.strMediaType;
  strSource := AMovie.strSource;
  iDate := AMovie.iDate;
  strBorrower := AMovie.strBorrower;
  iRating := AMovie.iRating;
  strOriginalTitle := AMovie.strOriginalTitle;
  strTranslatedTitle := AMovie.strTranslatedTitle;
  strDirector := AMovie.strDirector;
  strProducer := AMovie.strProducer;
  strCountry := AMovie.strCountry;
  strCategory := AMovie.strCategory;
  iYear := AMovie.iYear;
  iLength := AMovie.iLength;
  strActors := AMovie.strActors;
  strURL := AMovie.strURL;
  strDescription := AMovie.strDescription;
  strComments := AMovie.strComments;
  strVideoFormat := AMovie.strVideoFormat;
  iVideoBitrate := AMovie.iVideoBitrate;
  strAudioFormat := AMovie.strAudioFormat;
  iAudioBitrate := AMovie.iAudioBitrate;
  strResolution := AMovie.strResolution;
  strFramerate := AMovie.strFramerate;
  strLanguages := AMovie.strLanguages;
  strSubtitles := AMovie.strSubtitles;
  strSize := AMovie.strSize;
  iDisks := AMovie.iDisks;
  bChecked := AMovie.bChecked;
  strPicture := AMovie.strPicture;
  if (Picture <> nil) then
    FreeAndNil(Picture);
  if AMovie.Picture <> nil then
  begin
    Picture := TMemoryStream.Create;
    AMovie.Picture.Seek(0, soFromBeginning);
    Picture.CopyFrom(AMovie.Picture, AMovie.Picture.Size);
  end;
  if _thumb <> nil then
    FreeAndNil(_thumb);
  _thumbError := 0;
  _thumbWidth := 0;
  _thumbHeight := 0;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
procedure TMovie.SaveToXML(Root: TJvSimpleXmlElem; const strFileName: string = '');
var
  i: Integer;
  PicName: string;
begin
  if Root = nil then
    raise Exception.Create('TMovie.SaveToXML : nil element');
  for i := 0 to fieldCount-1 do
    AddNotEmpty(Root.Properties, strXmlFields[i], GetFieldValue(i));
  if (strPicture <> '') and (Picture <> nil) then
  begin
    if (strFileName <> '') then
    begin
      if Settings.rOptions.rMovieInformation.rPicImport.Naming = 2 then
        PicName := ValidateFileName(GetFormattedTitle) + strPicture
      else
        PicName := IntToStr(iNumber) + strPicture;
      if Settings.rOptions.rMovieInformation.rPicImport.CatalogPrefix then
        PicName := Format('%s_%s', [ChangeFileExt(ExtractFileName(strFileName), ''), PicName]);
      PicName := MakeUniqueFileName(PicName, '_');
      Picture.SaveToFile(PicName);
      PicName := ExtractFileName(PicName);
      strPicture := PicName;
    end
    else
      strPicture := '';
    Picture.Free;
    Picture := nil;
  end
  else
    PicName := strPicture;
  AddNotEmpty(Root.Properties, 'Picture', PicName);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovie.LoadFromXML(Root: TJvSimpleXmlElem; const Version: Integer = 99);
var
  i: Integer;
begin
  if Root = nil then
    raise Exception.Create('TMovie.LoadFromXML : nil element');
  for i := 0 to fieldCount-1 do
    SetFieldValue(i, ReadTag(Root.Properties, strXmlFields[i], GetFieldValue(i)));
  strPicture := ReadTag(Root.Properties, 'Picture', '');
//  if (Version < 35) and (iRating <> -1) then
//    iRating := iRating * 10;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovie.CanInclude(const IncOpt: TMovieIncludeOption): Boolean;
begin
  case IncOpt of
    mioSelected:    Result := Self._bSelected;
    mioChecked:     Result := Self.bChecked;
    mioVisible:     Result := Self._bVisible;
  else
    Result := True;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovie.CalcTotalSize: Int64;
var
  i: Integer;
  s: string;
begin
  i := 1;
  Result := 0;
  s := strSize;
  while i <= Length(s) do
  begin
    if s[i] in ['0'..'9'] then
      Inc(i)
    else
    begin
      if i > 1 then
        Inc(Result, StrToIntDef(Copy(s, 1, i - 1), 0));
      Delete(s, 1, i);
      i := 1;
    end;
  end;
  if i > 1 then
    Inc(Result, StrToIntDef(Copy(s, 1, i - 1), 0));
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovie.ContainsText(const Value: string; const Field: Integer; const WholeField: Boolean): Boolean;
var
  f: TMovieField;
begin
  Result := False;
  if Field in AllFields then
  begin
    Result := ((WholeField) and AnsiSameText(GetFieldValue(Field), Value))
      or ((not WholeField) and AnsiContainsText(GetFieldValue(Field), Value));
  end
  else
  if Field = -1 then
    for f := fieldLow to fieldCount-1 do
      if ContainsText(Value, f, WholeField) then
      begin
        Result := True;
        Exit;
      end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
function TMovie.GetPictureSize(const CatalogDir: TFileName = ''): Int64;
begin
  Result := 0;
  if strPicture <> '' then
  begin
    if Picture <> nil then
      Result := Picture.Size
    else
    begin
      if CatalogDir <> '' then
        SetCurrentDir(CatalogDir);
      Result := GetFileSize(ExpandFileName(strPicture));
    end;
  end;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
  TMovieList
-------------------------------------------------------------------------------}

constructor TMovieList.Create(AOwnsObjects: Boolean);
begin
  MovieProperties := TMovieProperties.Create;
  MovieProperties.InitFields;
  inherited Create(AOwnsObjects);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

destructor TMovieList.Destroy;
begin
  MovieProperties.Free;
  inherited;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.LoadFromFile(const strFileName: string);
var
  InputFile: TFileStream;
  Header: array[0..Length(strFileHeader)] of Char;
  iHeader{$IFNDEF DLLMode}, iRecSize{$ENDIF}: integer;
begin
  Header := '';
  iHeader := 0;
{$IFNDEF DLLMode}
  iRecSize := 0;
{$ENDIF}
  try
    InputFile:=TFileStream.Create(strFileName,fmOpenRead);
    try
      Inputfile.Seek(0,soFrombeginning);
      InputFile.Read(Header,Length(strFileHeader));
      if Header = strFileHeader35 then
      begin
        iHeader := 35;
      end else
      if Header = strFileHeader33 then
      begin
        iHeader := 33;
      end else
      if Header = strFileHeader31 then
      begin
        iHeader := 31;
{$IFNDEF DLLMode}
      end else
      if Header = strFileHeader30 then
      begin
        iHeader := 30;
        iRecSize := sizeof(RMovie30);
      end else
      if Header = strFileHeader21 then
      begin
        iHeader := 21;
        iRecSize := sizeof(RMovie21);
      end else
      if Header = strFileHeader11 then
      begin
        iHeader := 11;
        iRecSize := sizeof(RMovie11);
      end else
      if Header = strFileHeader10 then
      begin
        iHeader := 10;
        iRecSize := sizeof(RMovie10);
{$ENDIF}
      end;
      Clear;
      case iHeader of
{$IFNDEF DLLMode}
        10,11,21,30:
          begin
            ReadRecords(InputFile, iRecSize, iHeader);
            if iHeader < 30 then
            begin
              ReadPictures(ChangeFileExt(strFileName, ''));
              ReadBorrowers(ChangeFileExt(strFileName, '.amcl'));
            end;
          end;
{$ENDIF}
        31,33,35:
          begin
            ReadDatas(InputFile, iHeader);
          end;
      else
        raise Exception.Create(strUnableToIdentify);
      end; // case
    finally
      InputFile.Free;
    end; // try
  except
    on E: Exception do
      raise Exception.Create(Format(strErrorLoad,[strFilename])+E.Message);
  end; // try
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

class function TMovieList.ReadHeader(const AFileName: TFileName; out TheHeader: string): Int64;
var
  i: Integer;
begin
  with TFileStream.Create(AFileName, fmOpenRead) do
    try
      Seek(0, soFrombeginning);
      SetLength(TheHeader, Length(strFileHeader));
      Read(TheHeader[1], Length(strFileHeader));
      i := Pos(sLineBreak, TheHeader);
      if i > 0 then
        SetLength(TheHeader, i - 1);
      Result := Size;
    finally
      Free;
    end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
procedure TMovieList.LoadFromXML(const strFilename: string);
var
  i, iHeader: Integer;
  CurElem: TJvSimpleXmlElem;
  NewMovie: TMovie;
  HeaderItem: TJvSimpleXmlElemHeader;
begin
  Clear;
  with TJvSimpleXml.Create(nil) do
    try
      if Assigned(ProgressWin) then
        ProgressWin.Update; 
      LoadFromFile(strFilename);
      if Root.Name <> 'AntMovieCatalog' then
        raise Exception.Create(Format(strErrorLoad + strUnableToIdentify, [strFileName]));
      with Prolog do
      begin
        HeaderItem := nil;
        for i := 0 to Count-1 do
          if Item[i] is TJvSimpleXmlElemHeader then
          begin
            HeaderItem := TJvSimpleXmlElemHeader(Item[i]);
            Break;
          end;
        if HeaderItem <> nil then
          MovieProperties.strEncoding := HeaderItem.Encoding;
      end;
      iHeader := Root.Properties.IntValue('Format', intFileVersion);
      CurElem := Root.Items.ItemNamed['Catalog'];
      if CurElem = nil then
        raise Exception.Create(Format(strErrorLoad + strNoMovieFound, [strFileName]));
      CurElem := CurElem.Items.ItemNamed['Properties'];
      if CurElem <> nil then
        with CurElem, MovieProperties do
        begin
          strName := ReadTag(Properties, 'Owner', strName);
          strMail := ReadTag(Properties, 'Mail', strMail);
          strSite := ReadTag(Properties, 'Site', strSite);
          strDescription := ReadTag(Properties, 'Description', strDescription);
        end;
      CurElem := Root.Items.ItemNamed['Catalog'].Items.ItemNamed['Contents'];
      if CurElem <> nil then
        with CurElem do
        begin
          if Assigned(ProgressWin) then
          begin
            ProgressWin.Maximum := ProgressWin.Maximum + Ceil(Items.Count / 50) + 1;
            ProgressWin.StepIt;
          end;
          for i := 0 to Items.Count-1 do
          begin
            if (i mod 50 = 0) and Assigned(ProgressWin) then
              ProgressWin.StepIt;
            if Items.Item[i].Name = 'Movie' then
            begin
              NewMovie := TMovie.Create;
              with NewMovie do
              begin
                InitFields;
                NewMovie.LoadFromXML(Items.Item[i], iHeader);
              end;
              Self.Add(NewMovie);
            end;
          end;
        end;
    finally
      Free;
    end;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.ReadDatas(InputFile: TFileStream; const version: Integer);
var
  newMovie: TMovie;
  recsize: Integer;
  {$IFNDEF DLLMode}
  c: Integer;
  {$ENDIF}
  function ReadField: string;
  begin
    with InputFile do
    begin
      ReadBuffer(recsize, intsize);
      if recsize > 0 then
      begin
        SetLength(Result, recsize);
        ReadBuffer(Result[1], recsize);
      end else
        Result := '';
    end;
  end;
begin
  {$IFNDEF DLLMode}
  c := 0;
  if Assigned(ProgressWin) then
  begin
    ProgressWin.Maximum := Ceil(InputFile.Size / 1024) + 2;
    ProgressWin.IntProgress := 1;
  end;
  {$ENDIF}
  InputFile.Seek(Length(strFileHeader),soFrombeginning);
  with MovieProperties do
  begin
    strName := ReadField;
    strMail := ReadField;
    if version < 35 then
      ReadField; // ICQ field that was deleted
    strSite := ReadField;
    strDescription := ReadField;
  end;
  newMovie := nil;
  while InputFile.Position < InputFile.Size do
  begin
    {$IFNDEF DLLMode}
    Inc(c);
    if (c mod 50 = 0) and Assigned(ProgressWin) then
      ProgressWin.IntProgress := InputFile.Position div 1024;
    {$ENDIF}
    try
      newMovie := TMovie.Create;
      newMovie.ReadData(InputFile, version);
      Add(newMovie);
    except
      if IndexOf(NewMovie) = -1 then
        newMovie.Free;
      Break;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
procedure TMovieList.ReadRecords(InputFile: TFileStream; const RecordSize: longint; const version: integer);
var
  CurrentMovie: PMovie30;
  addedMovie: TMovie;
  ReadBytes: Longint;
  Header: array[0..Length(strFileHeader)] of Char;
  MovieProperties: RMovieProperties;
  ts: TTimeStamp;
begin
  Header := '';
  Inputfile.Seek(0,soFrombeginning);
  ReadBytes := InputFile.Read(Header,Length(strFileHeader));
  if (version >= 21) and (ReadBytes>0) then
  begin
    ReadBytes := InputFile.Read(MovieProperties,sizeof(RMovieProperties));
  end else
  begin
    with MovieProperties do
    begin
      strOwnerName := '';
      strOwnerMail := '';
      strOwnerICQ := '';
      strOwnerSite := '';
    end;
  end;
  with MovieProperties do
  begin
    Self.MovieProperties.strName := strOwnerName;
    Self.MovieProperties.strMail := strOwnerMail;
    Self.MovieProperties.strSite := strOwnerSite;
    Self.MovieProperties.strDescription := '';
  end;
  CurrentMovie := new(PMovie30);
  while(ReadBytes>0) do
  begin
    FillChar(CurrentMovie^,sizeof(CurrentMovie^),' ');
    ReadBytes := InputFile.Read(CurrentMovie^,RecordSize);
    with CurrentMovie^ do
    begin
      if(ReadBytes>0) then
      begin
        if version < 11 then
        begin
          iRating := 0;
        end;
        if version < 21 then
        begin
          bChecked := true;
          iDate := 0;
        end;
        if version < 30 then
        begin
          strPicture := '';
          iPictureSize := 0;
          strBorrower := '';
          if iDate = DefaultDate then
            iDate := 0;
        end;
        if version < 31 then
        begin
          case iRating of
            0: iRating := -1;
            1: iRating := 2;
            2: iRating := 4;
            3: iRating := 6;
            4: iRating := 8;
            5: iRating := 9;
          else
            iRating := -1;
          end;
          if iDate <> 0 then
          begin
            ts.Time := 0;
            ts.Date := iDate;
            iDate := trunc(TimeStampToDateTime(ts));
          end;
        end;
        addedMovie := TMovie.Create;
        addedMovie.SetFields(CurrentMovie);
        if iPictureSize > 0 then
        begin
          addedMovie.Picture := TMemoryStream.Create;
          addedMovie.Picture.CopyFrom(InputFile, iPictureSize);
        end;
        Add(addedMovie);
      end else
      begin
      end;
    end;
  end;
  Dispose(CurrentMovie);
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
procedure TMovieList.ReadPictures(const strFilename: string);
var
  i: integer;
  currentFile, ext: string;
begin
  for i := 0 to Count-1 do
  begin
    if Items[i] <> nil then
    begin
      with TMovie(Items[i]) do
      begin
        currentFile := Format('%s_%d', [strFileName, iNumber]);
        ext := '.jpg';
        if not FileExists(currentFile + ext) then
        begin
          ext := '.gif';
          if not FileExists(currentFile + ext) then
          begin
            ext := '.png';
            if not FileExists(currentFile + ext) then
            begin
              strPicture := '';
              continue;
            end; // not png
          end; // not gif
        end; // not jpg
        strPicture := ExtractFileName(currentFile) + ext;
      end;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.ReadBorrowers(const strFilename: string);
var
  Names: TStringList;
  Movies: TStringList;
  i,j: integer;
  Movie: TMovie;
begin
  with TMemIniFile.Create(strFileName) do
    try
      Names := TStringList.Create;
      Movies := TStringList.Create;
      try
        ReadSections(Names);
        for i := 0 to Names.Count-1 do
        begin
          ReadSection(Names.Strings[i], Movies);
          for j := 0 to Movies.Count-1 do
          begin
            Movie := Find(StrToInt(Movies.Strings[j]));
            if Movie <> nil then
            begin
              Movie.strBorrower := Names.Strings[i];
            end;
          end;
        end;
      finally
        Movies.Free;
        Names.Free;
      end;
    finally
      Free;
    end;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.SaveToFile(const strFilename: string);
var
  OutputFile: TFileStream;
  i, recsize: integer;

  procedure WriteField(const field: string);
  begin
    with OutputFile do
    begin
      recsize := length(field);
      WriteBuffer(recsize, intsize);
      if recsize > 0 then
        WriteBuffer(field[1], recsize);
    end;
  end;

begin
  {$IFNDEF DLLMode}
  if Assigned(ProgressWin) then
  begin
    ProgressWin.Maximum := ProgressWin.Maximum + Ceil(Count / 25) + 1;
    ProgressWin.StepIt;
  end;
  {$ENDIF}
  OutputFile := TFileStream.Create(strFileName,fmCreate);
  try
    OutputFile.Size := 0;
    try
      OutputFile.Write(strFileHeader35,Length(strFileHeader));
      with MovieProperties do
      begin
        WriteField(strName);
        WriteField(strMail);
        WriteField(strSite);
        WriteField(strDescription);
      end;
      for i := 0 to Count-1 do
      begin
        {$IFNDEF DLLMode}
        if (i mod 25 = 0) and Assigned(ProgressWin) then
          ProgressWin.StepIt;
        {$ENDIF}
        TMovie(Items[i]).WriteData(OutputFile);
      end;
    except
      on E: Exception do
        raise Exception.Create(Format(strErrorSave,[strFilename])+E.Message);
    end;
  finally
    OutputFile.Free;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
procedure TMovieList.SaveToXML(const strFilename: string; ConvertPictures: Boolean{; const Header: string});
var
  i: Integer;
  s: string;
  HeaderItem: TJvSimpleXmlElemHeader;
begin
  if Assigned(ProgressWin) then
  begin
    ProgressWin.Maximum := ProgressWin.Maximum + Ceil(Count / 25);
    ProgressWin.StepIt;
  end;
  if ConvertPictures then
    s := strFilename
  else
    s := '';
  with TJvSimpleXml.Create(nil) do
    try
      WriteXMLHeader(Root, intFileVersion, 'AntMovieCatalog', strVersion, strDate);
      with Prolog do
      begin
        HeaderItem := nil;
        for i := 0 to Count-1 do
          if Item[i] is TJvSimpleXmlElemHeader then
          begin
            HeaderItem := TJvSimpleXmlElemHeader(Item[i]);
            Break;
          end;
        if HeaderItem = nil then
        begin
          HeaderItem := TJvSimpleXmlElemHeader.Create;
          Add(HeaderItem);
        end;
        HeaderItem.Encoding := MovieProperties.strEncoding;
      end;
      Root.Items.Delete('Catalog');
      with Root.Items.Add('Catalog') do
      begin
        with Items.Add('Properties'), MovieProperties do
        begin
          AddNotEmpty(Properties, 'Owner', strName);
          AddNotEmpty(Properties, 'Mail', strMail);
          AddNotEmpty(Properties, 'Site', strSite);
          AddNotEmpty(Properties, 'Description', strDescription);
        end;
        with Items.Add('Contents') do
        begin
          SetCurrentDir(ExtractFilePath(strFilename)); // for images
          for i := 0 to Count-1 do
          begin
            if (i mod 25 = 0) and Assigned(ProgressWin) then
              ProgressWin.StepIt;
            TMovie(Self.Items[i]).SaveToXML(Items.Add('Movie'), s);
          end;
        end;
      end;
      if Assigned(ProgressWin) then
        ProgressWin.StepIt;
      SaveToFile(strFilename);
    finally
      Free;
    end;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.Sort(const Field: Integer);
begin
  SortField := Field;
  inherited Sort(CompareStd);
end;

procedure TMovieList.SortReverse(const Field: Integer);
begin
  SortField := Field;
  inherited Sort(CompareStdReverse);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
procedure TMovieList.Sort(const FieldsList: TStrings);
begin
  SortFieldsList.Assign(FieldsList);
  inherited Sort(CompareAdv);
end;

procedure TMovieList.SortReverse(const FieldsList: TStrings);
begin
  SortFieldsList.Assign(FieldsList);
  inherited Sort(CompareAdvReverse);
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.Find(const MovieNumber: Integer): TMovie;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count-1 do
  begin
    if TMovie(Items[i]).iNumber = MovieNumber then
    begin
      Result := Items[i];
      Break;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.Add(const AMovieList: TMovieList; const AllowDuplicateNumbers: Boolean);
var
  i: Integer;
  NewMovie, CurrentMovie: TMovie;
begin
  if (AMovieList <> nil) and (AMovieList.Count > 0) then
  begin
    AMovieList.Sort(fieldNumber);
    for i := 0 to AMovieList.Count-1 do
    begin
      CurrentMovie := TMovie(AMovieList.Items[i]);
      NewMovie := TMovie.Create;
      if (CurrentMovie.iNumber < 1) or ((not AllowDuplicateNumbers) and (Find(CurrentMovie.iNumber) <> nil)) then
        NewMovie.iNumber := MaxNumber(False) + 1
      else
        NewMovie.iNumber := CurrentMovie.iNumber;
      NewMovie.Assign(CurrentMovie, False);
      inherited Add(NewMovie);
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.Add: TMovie;
begin
  Result := TMovie.Create;
  inherited Add(Result);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.Add2: IMovie;
begin
  Result := Add;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.MaxNumber(ListIsSorted: Boolean = False): Integer;
begin
  if not ListIsSorted then
    Sort(fieldNumber);
  if Count = 0 then
    Result := 0
  else
    Result := TMovie(Items[Count-1]).iNumber;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.FirstFreeNumber(ListIsSorted: Boolean = False): Integer;
var
  i, Prev: Integer;
begin
  if not ListIsSorted then
    Sort(fieldNumber);
  Prev := 0;
  for i := 0 to Count-1 do
  begin
    if TMovie(Items[i]).iNumber > (Prev + 1) then
      Break
    else
      Prev := TMovie(Items[i]).iNumber;
  end;
  Result := Prev + 1;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.HasImages(IncludeStored, IncludeLinksRel, IncludeLinksAbs: Boolean): Boolean;
var
  i: Integer;
  PicStr, PicDrive: Boolean;
begin
  Result := False;
  for i := 0 to Count-1 do
  begin
    with TMovie(Items[i]) do
    begin
      PicStr := strPicture <> '';
      PicDrive := ExtractFileDrive(strPicture) <> '';
      Result := (IncludeStored and (Picture <> nil) and PicStr) or
                ((Picture = nil)
                         and PicStr
                         and ((IncludeLinksRel and not PicDrive) or
                              (IncludeLinksAbs and PicDrive)));
    end;
    if Result then
      Break;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.GetValues(AList: TStrings; Field: Integer);
var
  i: Integer;
  value: string;
begin
  AList.BeginUpdate;
  AList.Clear;
  try
    for i := 0 to Count-1 do
    begin
      value := TMovie(Items[i]).GetFieldValue(Field);
      if (value <> '') and (AList.IndexOf(value) = -1) then
        AList.Add(value);
    end;
  finally
    AList.EndUpdate;
  end;
end;

{-------------------------------------------------------------------------------
  TMovieProperties
-------------------------------------------------------------------------------}

procedure TMovieProperties.InitFields;
begin
  strEncoding := 'iso-8859-1';
  strName := '';
  strMail := '';
  strSite := '';
  strDescription := '';
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.Count: Integer;
begin
  Result := inherited Count;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.Count(const IncOpt: TMovieIncludeOption): Integer;
var
  i: Integer;
begin
  Result := 0;
  for i := 0 to Count-1 do
    if TMovie(Items[i]).CanInclude(IncOpt) then
      Inc(Result);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.Count(out cAll, cSelected, cChecked, cVisible: Integer);
var
  i: Integer;
begin
  cAll := 0;
  cSelected := 0;
  cChecked := 0;
  cVisible := 0;
  for i := 0 to Count-1 do
    with TMovie(Items[i]) do
    begin
      Inc(cAll);
      if _bSelected then
        Inc(cSelected);
      if bChecked then
        Inc(cChecked);
      if _bVisible then
        Inc(cVisible);
    end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.Count(const MovieNumber: Integer): Integer;
var
  i: Integer;
begin
  Result := 0;
  for i := 0 to Count-1 do
    with TMovie(Items[i]) do
      if iNumber = MovieNumber then
        Inc(Result);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.MakeFilteredList(AList: TList; const IncOpt: TMovieIncludeOption);
var
  i: Integer;
begin
  AList.Clear;
  for i := 0 to Count-1 do
    if TMovie(Items[i]).CanInclude(IncOpt) then
      AList.Add(Items[i]);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.ShiftNumbers(const StartAt: Integer; const Exclude: TMovie);
var
  i: Integer;
begin
  Sort(fieldNumber);
  for i := 0 to Count-1 do
    if Exclude <> Items[i] then
      with TMovie(Items[i]) do
        if iNumber >= StartAt then
        begin
          Inc(iNumber);
          if (i < Count-1) and ((TMovie(Items[i+1]).iNumber > iNumber) or (TMovie(Items[i+1]) = Exclude)) then
            Exit;
        end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.GetItem(const idx: Integer): TMovie;
begin
  Result := TMovie(inherited Items[idx]);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function TMovieList.GetItem2(const idx: Integer): IMovie;
begin
  Result := GetItem(idx);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TMovieList.SetItem(const idx: Integer; const Value: TMovie);
begin
  inherited Items[idx] := Value;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
function TMovieList.GetMoviesByGroups(const Field: Integer; const IncOpt: TMovieIncludeOption = mioALL;
      MovieFilter : TMovieFilter = nil): TStringList;
var
  Groups: TStringList;
  GroupMovies: TObjectList;
  Movie: TMovie;
  FieldValue, GroupName: string;
  Sep : Char;
  RmPFields : TMovieFields;
  i, len, idx, n, initPos: Integer;
  nbPOpen, firstPOpen, lastPClose: Integer;
  bGroupMulti, bPatch : Boolean;

  // Args: IN [GroupName, Movie]
  procedure AddMovieToMap;
  begin
    if Length(GroupName) > 128 then
      GroupName := Copy(GroupName, 1, 128);
    GroupName := Trim(GroupName);
    if GroupName <> '' then
    begin
      idx := Groups.IndexOf(GroupName);
      if idx = -1 then
      begin
        GroupMovies := TObjectList.Create;
        GroupMovies.Add(Movie);
        Groups.AddObject(GroupName, GroupMovies);
      end else
      begin
        GroupMovies := TObjectList(Groups.Objects[idx]);
        if(GroupMovies.IndexOf(Movie) = -1) then
          GroupMovies.Add(Movie);
        //Groups.Objects[idx] := GroupMovies;
      end;
    end;
  end;
begin
  with Settings.rOptions.rMovieList do
  begin
    bGroupMulti := GroupMulti;
    Sep := GroupMultiSep;
    bPatch := GroupMultiAddPatch;
    if GroupMultiRmAllP then
      RmPFields := GroupByFieldsMulti
    else
      RmPFields := GroupByFieldsMultiDefaultRmP;
  end;
  Groups := TStringList.Create;
  Groups.Sorted := True;
  Groups.CaseSensitive := False;
  
  for i := 0 to Count-1 do
  begin
    Movie := TMovie(Items[i]);
    if not Movie.CanInclude(IncOpt) then
      continue;
    if MovieFilter <> nil then
      with MovieFilter do
      begin
        if not Movie.ContainsText(value, field, wholeField) then
          continue;
      end;
    with Movie do
    begin
      if Field in [fieldLow..fieldCount-1] then
      begin
        FieldValue := GetFieldValue(Field);
        len := Length(FieldValue);
        if (len > 0) and (FieldValue[len] = '.') then
          FieldValue := Copy(FieldValue, 1, len-1);
        if (FieldValue = '') then
          FieldValue := '$$$EMPTY$$$';
      end else
      begin
        FieldValue := '$$$NOGROUP$$$';
      end;
      if (not bGroupMulti) or (not (Field in GroupByFieldsMulti)) then
      begin
        GroupName := FieldValue;
        AddMovieToMap;
      end else
      begin
        initPos := 1;
        n := 1;
        nbPOpen := 0;
        firstPOpen := 0;
        lastPClose := 0;
        while True do
        begin
          if (FieldValue[n] = ')') then
          begin
            Dec(nbPOpen);
            if nbPOpen = 0 then
              lastPClose := n;
          end else if (FieldValue[n] = '(') then
          begin
            if(not bPatch) or (not
              (((FieldValue[n+1] = 'I') or (FieldValue[n+1] = 'V') or (FieldValue[n+1] = 'X')) and
               ((FieldValue[n+2] = 'I') or (FieldValue[n+2] = 'V') or (FieldValue[n+2] = 'X') or
                (FieldValue[n+2] = Sep) or (FieldValue[n+2] = #10) or
                ((FieldValue[n+2] = ' ') and (FieldValue[n+3] = '('))) and
               ((FieldValue[n+2] = Sep) or (FieldValue[n+2] = #10) or
                ((FieldValue[n+2] = ' ') and (FieldValue[n+3] = '(')) or
                (FieldValue[n+3] = 'I') or (FieldValue[n+3] = 'V') or (FieldValue[n+3] = 'X') or
                (FieldValue[n+3] = Sep) or (FieldValue[n+3] = #10) or
                ((FieldValue[n+3] = ' ') and (FieldValue[n+4] = '('))) and
               ((FieldValue[n+2] = Sep) or (FieldValue[n+2] = #10) or
                ((FieldValue[n+2] = ' ') and (FieldValue[n+3] = '(')) or (FieldValue[n+3] = Sep) or
                (FieldValue[n+3] = #10) or ((FieldValue[n+3] = ' ') and (FieldValue[n+4] = '(')) or
                (FieldValue[n+4] = Sep) or (FieldValue[n+4] = #10) or
                ((FieldValue[n+4] = ' ') and (FieldValue[n+5] = '('))))) then
            begin
              if (nbPOpen = 0) then
                firstPOpen := n;
              Inc(nbPOpen);
            end;
          end else if (FieldValue[n] = Sep) or (FieldValue[n] = #0) then
          begin
            if nbPOpen = 0 then // Good parentheses
            begin
              if(firstPOpen > 0) and (Field in RmPFields) then // There are Parentheses
              begin
                GroupName := Trim(Copy(FieldValue, initPos, firstPOpen - initPos)) +' '+
                             Trim(Copy(FieldValue, lastPClose + 1, n - lastPClose - 1));
              end else
              begin
                GroupName := Copy(FieldValue, initPos, n - initPos);
              end;
              AddMovieToMap;
              initPos := n + 1;
              firstPOpen := 0;
              lastPClose := 0;
            end else if(FieldValue[n] = #0) then // Bad parentheses !
            begin
              GroupName := strErrorParenthesis; // Copy(FieldValue, initPos, n - initPos);
              AddMovieToMap;
            end;
            if(FieldValue[n] = #0) then // Finish
              break;
          end;
          Inc(n);
        end; // while True
      end; // else GroupByFieldsMulti
    end; // end with Movie
  end; // end for
  Result := Groups;
end;
{$ENDIF}

{-------------------------------------------------------------------------------
    External functions
-------------------------------------------------------------------------------}

function GetFieldType(field: Integer): TFieldType;
begin
  case field of
    fieldNumber,
    fieldYear,            fieldLength,          fieldVideoBitrate,
    fieldAudioBitrate,    fieldDisks:
      Result := ftInteger;
    fieldRating:
      Result := ftReal;
    fieldDate:
      Result := ftDate;
    fieldChecked,
    fieldMedia,           fieldMediaType,       fieldSource,
    fieldBorrower,        fieldOriginalTitle,
    fieldTranslatedTitle, fieldFormattedTitle,
    fieldDirector,        fieldProducer,
    fieldCountry,         fieldCategory,        fieldURL,
    fieldVideoFormat,     fieldAudioFormat,     fieldResolution,
    fieldFrameRate,       fieldLanguages,       fieldSubtitles,
    fieldSize:
      Result := ftString;
    fieldActors,          fieldDescription,     fieldComments:
      Result := ftList;
  else
    Result := ftOther;
  end; // case
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function CompareStd(Item1, Item2: Pointer): Integer;
var
  Movie1, Movie2: TMovie;
begin
  Result := 0;
  Movie1 := Item1;
  Movie2 := Item2;
  case GetFieldType(SortField) of
    ftInteger, ftDate:
        Result := Movie1.GetIntFieldValue(SortField) - Movie2.GetIntFieldValue(SortField);
    ftReal:
      if SortField = fieldRating then
        Result := Movie1.GetIntFieldValue(SortField) - Movie2.GetIntFieldValue(SortField);
    ftString:
        Result := AnsiCompareText(Movie1.GetFieldValue(SortField), Movie2.GetFieldValue(SortField));
  end; // case
end;

function CompareStdReverse(Item1, Item2: Pointer): Integer;
begin
  Result := -CompareStd(Item1, Item2);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

{$IFNDEF DLLMode}
function CompareAdv(Item1, Item2: Pointer): Integer;
var
  Movie1, Movie2: TMovie;
  field: Integer;
begin
  Result := 0;
  try
    Movie1 := Item1;
    Movie2 := Item2;
    Inc(SortFieldsIndex);
    if SortFieldsIndex < SortFieldsList.Count then
    begin
      field := strFields.IndexOf(SortFieldsList[SortFieldsIndex]);
      case GetFieldType(field) of
        ftInteger, ftDate:
          begin
            Result := Movie1.GetIntFieldValue(field) - Movie2.GetIntFieldValue(field);
            if Result = 0 then
              Result := CompareAdv(Item1, Item2);
          end;
        ftReal:
          begin
            if field = fieldRating then
              Result := Movie1.GetIntFieldValue(field) - Movie2.GetIntFieldValue(field);
            if Result = 0 then
              Result := CompareAdv(Item1, Item2);
          end;
        ftString:
          begin
            Result := AnsiCompareText(Movie1.GetFieldValue(field), Movie2.GetFieldValue(field));
            if Result = 0 then
              Result := CompareAdv(Item1, Item2);
          end;
      end; // case
    end; // if
  finally
    SortFieldsIndex := -1;
  end; // try
end;

function CompareAdvReverse(Item1, Item2: Pointer): Integer;
var
  Movie1, Movie2: TMovie;
  field: Integer;
begin
  Result := 0;
  try
    Movie1 := Item1;
    Movie2 := Item2;
    Inc(SortFieldsIndex);
    if SortFieldsIndex < SortFieldsList.Count then
    begin
      field := strFields.IndexOf(SortFieldsList[SortFieldsIndex]);
      case GetFieldType(field) of
        ftInteger, ftDate:
          begin
            Result := Movie1.GetIntFieldValue(field) - Movie2.GetIntFieldValue(field);
            if Result = 0 then
              Result := CompareAdvReverse(Item1, Item2);
          end;
        ftReal:
          begin
            if field = fieldRating then
              Result := Movie1.GetIntFieldValue(field) - Movie2.GetIntFieldValue(field);
            if Result = 0 then
              Result := CompareAdvReverse(Item1, Item2);
          end;
        ftString:
          begin
            Result := AnsiCompareText(Movie1.GetFieldValue(field), Movie2.GetFieldValue(field));
            if Result = 0 then
              Result := CompareAdvReverse(Item1, Item2);
          end;
      end; // case
    end; // if
    Result := -Result;
  finally
    SortFieldsIndex := -1;
  end; // try
end;
{$ENDIF}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

initialization
{$IFDEF MSWINDOWS}
  ClipboardFormat := RegisterClipBoardFormat(strClipboardHeader);
{$ENDIF}
{$IFNDEF DLLMode}
  SortFieldsList := TStringList.Create;
  SortFieldsIndex := -1;
{$ENDIF}
finalization
{$IFNDEF DLLMode}
  SortFieldsList.Free;
{$ENDIF}
end.

